using System;
using System.Data;
using Microsoft.SqlServer.Server;
using System.Data.SqlTypes;
using System.IO;
using System.Text;
using System.Collections.Generic;

//Needs to pass binary data to SQL
[Serializable]
//Defines the contract for Init, Accumilate, Merge and Terminate
[SqlUserDefinedAggregate(Format.UserDefined, MaxByteSize = 8000)]
public class Percentile : IBinarySerialize
{
    private List<SqlDouble> AccumilatedData;
    private SqlDouble Result;
    private SqlDouble PercentileNth;

    public void Init() //Initialize call to aggregate
    {
        AccumilatedData = new List<SqlDouble>();
        PercentileNth = 0;
        Result = 0;
    }

    //Called on every row that is passed to the aggregate
    public void Accumulate(SqlDouble value, SqlDouble pNth)
    {
        if (value.IsNull || pNth.IsNull)
        {
            return;
        }

        //Accumilate each row to a list
        AccumilatedData.Add(value.Value);
        PercentileNth = pNth;
    }

    //Called if another type of Percentile is merged with this one
    public void Merge(Percentile other)
    {
        AccumilatedData.AddRange(other.AccumilatedData);
    }

    //Called at the end of the aggregate to return the result
    public SqlDouble Terminate()
    {
        return Result;
    }

    //Called before terminate to read the temporarily stored data back to the aggregate
    public void Read(BinaryReader r)
    {
        Result = r.ReadDouble();
        PercentileNth = r.ReadDouble();
    }

    //Writes the data to temporary storage
    public void Write(BinaryWriter w)
    {
        PercentileCalculator pc = new PercentileCalculator();
        w.Write((Double)pc.GetPercentileCalculator(AccumilatedData, PercentileNth));
    }
}